#include<stdio.h>
#include<X11/Xlib.h>

#include"xyag.h"


/* オブジェクト描画 */

void draw_obj( obj_t *data )
{
  XDrawLines( Disp,Scr_p,Gc,data->data,data->length,CoordModeOrigin );
}


/* ページ描画 */

void draw_page( page_t *data )
{
  obj_t *obj;

  for( obj = data->data_head.next ; obj != NULL  ; obj = obj->next )
    draw_obj( obj );
}


/* 画面消去 */

void clear_scr( void )
{
  XFillRectangle( Disp,Scr_p,Gc_bg,0,0,Scr_width,Scr_height );
}


/* 画面更新 */

void flush_scr(void )
{

  XCopyArea( Disp,Scr_p,Scr_w,Gc_fg,0,0,Scr_width,Scr_height,0,0 );
}
